( cd usr/lib64 ; rm -rf libguilereadline-v-17.so )
( cd usr/lib64 ; ln -sf libguilereadline-v-17.so.17.0.3 libguilereadline-v-17.so )
( cd usr/lib64 ; rm -rf libguile-srfi-srfi-4-v-3.so.3 )
( cd usr/lib64 ; ln -sf libguile-srfi-srfi-4-v-3.so.3.0.1 libguile-srfi-srfi-4-v-3.so.3 )
( cd usr/lib64 ; rm -rf libguilereadline-v-17.so.17 )
( cd usr/lib64 ; ln -sf libguilereadline-v-17.so.17.0.3 libguilereadline-v-17.so.17 )
( cd usr/lib64 ; rm -rf libguile-srfi-srfi-4-v-3.so )
( cd usr/lib64 ; ln -sf libguile-srfi-srfi-4-v-3.so.3.0.1 libguile-srfi-srfi-4-v-3.so )
( cd usr/lib64 ; rm -rf libguile.so )
( cd usr/lib64 ; ln -sf libguile.so.17.2.0 libguile.so )
( cd usr/lib64 ; rm -rf libguile-srfi-srfi-60-v-2.so.2 )
( cd usr/lib64 ; ln -sf libguile-srfi-srfi-60-v-2.so.2.0.2 libguile-srfi-srfi-60-v-2.so.2 )
( cd usr/lib64 ; rm -rf libguile-srfi-srfi-1-v-3.so.3 )
( cd usr/lib64 ; ln -sf libguile-srfi-srfi-1-v-3.so.3.0.2 libguile-srfi-srfi-1-v-3.so.3 )
( cd usr/lib64 ; rm -rf libguile-srfi-srfi-1-v-3.so )
( cd usr/lib64 ; ln -sf libguile-srfi-srfi-1-v-3.so.3.0.2 libguile-srfi-srfi-1-v-3.so )
( cd usr/lib64 ; rm -rf libguile-srfi-srfi-13-14-v-3.so )
( cd usr/lib64 ; ln -sf libguile-srfi-srfi-13-14-v-3.so.3.0.1 libguile-srfi-srfi-13-14-v-3.so )
( cd usr/lib64 ; rm -rf libguile-srfi-srfi-60-v-2.so )
( cd usr/lib64 ; ln -sf libguile-srfi-srfi-60-v-2.so.2.0.2 libguile-srfi-srfi-60-v-2.so )
( cd usr/lib64 ; rm -rf libguile-srfi-srfi-13-14-v-3.so.3 )
( cd usr/lib64 ; ln -sf libguile-srfi-srfi-13-14-v-3.so.3.0.1 libguile-srfi-srfi-13-14-v-3.so.3 )
( cd usr/lib64 ; rm -rf libguile.so.17 )
( cd usr/lib64 ; ln -sf libguile.so.17.2.0 libguile.so.17 )
